package com.tofly.system.controller;


import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.result.BaseController;
import com.tofly.common.core.result.ResultData;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.entity.dto.BaseDto;
import com.tofly.system.entity.SysUser;
import com.tofly.system.entity.dto.UserDto;
import com.tofly.system.entity.query.UserQuery;
import com.tofly.system.entity.vo.user.UserVo;
import com.tofly.system.service.SysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.CacheManager;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * 系统用户(SysUser)表控制层
 *
 * @author HaiQiu
 * @since 2022-08-01 14:44:34
 */
@Api(tags = "系统用户")
@RestController
@RequestMapping("sysUser")
public class SysUserController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private SysUserService sysUserService;

    /**
     * 分页查询所有数据
     *
     * @param page 分页对象
     * @param userQuery 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据")
    @GetMapping
    public ResultData pageList(Page page, UserQuery userQuery) {
        return decide(sysUserService.pageList(page, userQuery));
    }

    /**
     * 查询所有数据
     *
     * @param userQuery 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "所有数据")
    @GetMapping("/allList")
    public ResultData allList(UserQuery userQuery) {
        return decide(sysUserService.allList(userQuery));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id",value = "数据ID")
    @GetMapping("{id}")
    public ResultData selectOne(@PathVariable Long id) {
        return decide(sysUserService.get(id));
    }

    /**
     * 新增数据
     *
     * @param userDto 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultData insert(@RequestPart @Validated(value = BaseDto.Add.class) UserDto userDto,
                             @RequestPart(required = false) @RequestParam(name = "avatarFile",required = false) MultipartFile avatarFile) {
        sysUserService.insert(userDto,avatarFile);
        return ResultData.ok(ResultEnum.ADD_SUCCESS);
    }

    /**
     * 修改数据
     *
     * @param userDto 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultData update(@RequestPart @Validated(value = BaseDto.Update.class) UserDto userDto,
                             @RequestPart(required = false) @RequestParam(name = "avatarFile",required = false) MultipartFile avatarFile) {
        sysUserService.edit(userDto,avatarFile);
        return ResultData.ok(ResultEnum.UPDATE_SUCCESS);
    }

    /**
     * 修改密码
     * @param oldPwd 旧密码
     * @param newPwd 新密码
     * @param newRePwd 重复新密码
     */
    @ApiOperation(value = "修改密码")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "oldPwd", value = "旧密码",dataType = "string",required = true),
            @ApiImplicitParam(name = "newPwd", value = "新密码",dataType = "string",required = true),
            @ApiImplicitParam(name = "newRePwd", value = "重复新密码",dataType = "string",required = true),
    })
    @PutMapping("/update/pwd")
    public ResultData updatePwd(@RequestParam(required = false) String oldPwd,
                               @RequestParam String newPwd,
                               @RequestParam String newRePwd) {
        sysUserService.updatePwd(oldPwd,newPwd,newRePwd);
        return ResultData.ok(ResultEnum.UPDATE_SUCCESS);
    }

    @ApiOperation(value = "重置密码")
    @PutMapping("/reset/pwd")
    public ResultData resetPwd(@RequestParam Set<Long> userIds) {
        // 用户默认密码六个零的hash值
        String defaultPwd = SysConstants.getUserDefaultPwd();
        boolean update = sysUserService.update(Wrappers.<SysUser>lambdaUpdate()
                .set(SysUser::getPassword, defaultPwd)
                .in(!CollectionUtils.isEmpty(userIds), SysUser::getId, userIds));
        //清空登录缓存
        List<Long> ids = new ArrayList<>(userIds);
        sysUserService.clearCached(ids);
        return ResultData.ok(update ? "重置成功，新密码：Zmgc@2024" : ResultEnum.UPDATE_FAIL);
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList",value = "数据ID数组集合")
    @DeleteMapping
    public ResultData delete(@RequestParam("idList") List<Long> idList) {
        sysUserService.delete(idList);
        return ResultData.ok(ResultEnum.DELETE_SUCCESS);
    }
    
    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        sysUserService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultData importData(@RequestParam(name = "file") @RequestPart MultipartFile file) {
        sysUserService.importData(file);
        return ResultData.ok("导入成功");
    }


    /**
     * 数据导出
     * @param page 分页
     * @param userQuery 查询模型
     * @param isAll 是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page page, UserQuery userQuery, Boolean isAll) {
        sysUserService.export(page,userQuery,isAll);
    }
}

