package com.tofly.system.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.oauth.auth.Dept;
import com.tofly.common.oauth.auth.Tenant;
import com.tofly.common.oauth.auth.ToflyUser;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.common.service.IFileInfoService;
import com.tofly.system.entity.SysApp;
import com.tofly.system.entity.SysAppPermission;
import com.tofly.system.entity.dto.AppDto;
import com.tofly.system.mapper.SysAppMapper;
import com.tofly.system.service.SysAppPermissionService;
import com.tofly.system.service.SysAppService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author haiqiu
 * @description 针对表【SYS_APP(系统App管理)】的数据库操作Service实现
 * @createDate 2022-08-01 14:50:59
 */
@Service
public class SysAppServiceImpl extends ServiceImpl<SysAppMapper, SysApp>
        implements SysAppService {

    @Value("${user.appPath}")
    private String appPath;

    @Autowired
    private SysAppPermissionService appPermissionService;

    @Autowired
    private IFileInfoService fileInfoService;

    @Override
    @Transactional
    public void insert(AppDto appDto, List<MultipartFile> fileList) {
        SysApp sysApp = new SysApp();
        //文件附件上传存储
        setDefaultVal(appDto, fileList, sysApp);
        //上传时间
        sysApp.setUploadUser(SecurityUtils.getUserId());
        sysApp.setUploadTime(new Date());
        if (!save(sysApp)) {
            throw new RuntimeException(ResultEnum.ADD_FAIL.getMessage());
        }
        // 发布范围
        if (!ObjectUtils.isEmpty(appDto.getAppPermission())) {
            appPermissionService.save(appDto.getAppPermission());
        }
    }


    @Transactional
    @Override
    public void edit(AppDto appDto, List<MultipartFile> fileList) {
        SysApp sysApp = getById(appDto.getId());
        setDefaultVal(appDto, fileList, sysApp);
        if (!updateById(sysApp)) {
            throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
        }
        // 发布范围
        if (!ObjectUtils.isEmpty(appDto.getAppPermission())) {
            appPermissionService.remove(Wrappers.<SysAppPermission>lambdaQuery().eq(SysAppPermission::getAppId, sysApp.getId()));
            appPermissionService.save(appDto.getAppPermission());
        }
    }

    private void setDefaultVal(AppDto appDto, List<MultipartFile> fileList, SysApp sysApp) {
        //文件附件上传存储
        String uploadFiles = fileInfoService.uploadFiles(fileList, appDto.getAnnex(), sysApp.getAnnex(), appPath, SysConstants.SYS_APP);
        appDto.setAnnex(uploadFiles);
        BeanUtils.copyProperties(appDto, sysApp);
        //发布时间
        if (sysApp.getStatus().equals(SysConstants.SYS_APP_RELEASE)){
            sysApp.setReleaseTime(new Date());
            sysApp.setReleaseUser(SecurityUtils.getUserId());
        }
    }

    @Override
    public SysApp getUpdateApp() {
        ToflyUser user = SecurityUtils.getUser();
        //未登录获取新版本
        if (user == null) {
            Page<SysApp> page = page(new Page<>(1,1),Wrappers.<SysApp>lambdaQuery().eq(SysApp::getStatus, 1).orderByDesc(SysApp::getUpdateTime));
            if (!CollectionUtils.isEmpty(page.getRecords())) {
                page.getRecords().get(0).setFileInfos(fileInfoService.getFIleByJsonData(page.getRecords().get(0).getAnnex()));
                return page.getRecords().get(0);
            }
        }
        //获取已登录版本
        Set<Long> appIds = getAppIds(user);
        Page<SysApp> page = page(new Page<>(1,1),(Wrappers.<SysApp>lambdaQuery()
                .eq(SysApp::getStatus, 1).orderByDesc(SysApp::getUpdateTime)
                .in(!CollectionUtils.isEmpty(appIds),SysApp::getId,appIds)));
        if (!CollectionUtils.isEmpty(page.getRecords())) {
            page.getRecords().get(0).setFileInfos(fileInfoService.getFIleByJsonData(page.getRecords().get(0).getAnnex()));
            return page.getRecords().get(0);
        }
        return null;
    }


    /**
     * 该方法返回app发布权限范围数据的appID查询
     * @param user 登录用户
     * @return appId集合
     */
    private Set<Long> getAppIds(ToflyUser user) {
        Tenant tenant = user.getTenant();
        List<Dept> depts = user.getDepts();
        Long userId = user.getId();
        Set<Long> appIds = new HashSet<>();
        if (!CollectionUtils.isEmpty(depts)){
            for (Dept dept : depts) {
                List<SysAppPermission> permissions = appPermissionService.list(Wrappers.<SysAppPermission>lambdaQuery()
                        .and(q -> q.like(tenant != null && tenant.getId() != null, SysAppPermission::getTenantId, tenant.getId())
                                .or()
                                .like(userId != null, SysAppPermission::getUserId, userId)
                                .or()
                                .like(!ObjectUtils.isEmpty(dept), SysAppPermission::getDeptId, dept.getId()))
                        .select(SysAppPermission::getAppId));
                if (!CollectionUtils.isEmpty(permissions)){
                    List<Long> collect = permissions.stream().map(s-> Long.parseLong(s.getAppId())).collect(Collectors.toList());
                    appIds.addAll(collect);
                }
            }
        }else {
            List<SysAppPermission> permissions = appPermissionService.list(Wrappers.<SysAppPermission>lambdaQuery()
                    .and(q -> q.like(tenant != null && tenant.getId() != null, SysAppPermission::getTenantId, tenant.getId())
                            .or()
                            .like(userId != null, SysAppPermission::getUserId, userId))
                    .select(SysAppPermission::getAppId));
            if (!CollectionUtils.isEmpty(permissions)){
                List<Long> collect = permissions.stream().map(s-> Long.parseLong(s.getAppId())).collect(Collectors.toList());
                appIds.addAll(collect);
            }
        } return appIds;
    }

    @Transactional
    @Override
    public void delete(List<Long> idList) {
        removeByIds(idList);
        //删除关联的发布范围
        appPermissionService.remove(Wrappers.<SysAppPermission>lambdaQuery().in(SysAppPermission::getAppId, idList));
    }

    @Override
    public SysApp get(Serializable id) {
        SysApp sysApp = getById(id);
        List<SysAppPermission> sysAppPermissions = appPermissionService.list(Wrappers.<SysAppPermission>lambdaQuery().eq(SysAppPermission::getAppId, id));
        if (!CollectionUtil.isEmpty(sysAppPermissions)){
            sysApp.setAppPermission(sysAppPermissions.get(0));
        }
        sysApp.setFileInfos(fileInfoService.getFIleByJsonData(sysApp.getAnnex()));
        return sysApp;
    }
}




