package com.tofly.system.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.system.entity.*;
import com.tofly.system.entity.SysMenu;
import com.tofly.system.entity.dto.MenuDto;
import com.tofly.system.entity.vo.menu.MenuVo;
import com.tofly.system.service.*;
import com.tofly.system.mapper.SysMenuMapper;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author haiqiu
 * @description 针对表【SYS_MENU(系统菜单)】的数据库操作Service实现
 * @createDate 2022-08-01 14:51:16
 */
@Service
@AllArgsConstructor
public class SysMenuServiceImpl extends ServiceImpl<SysMenuMapper, SysMenu>
        implements SysMenuService {

    private final SysMenuTenantService menuTenantService;

    private final SysRoleMenuService roleMenuService;

    private final SysUserService sysUserService;

    @Override
    @Transactional
    public void insert(MenuDto menuDto) {
        SysMenu sysMenu = new SysMenu();
        BeanUtils.copyProperties(menuDto, sysMenu);
        if (!save(sysMenu)) {
            throw new RuntimeException(ResultEnum.ADD_FAIL.getMessage());
        }
        saveMenuTenant(sysMenu.getId(), menuDto.getTenantIds());
    }

    @Override
    @Transactional
    public void edit(MenuDto menuDto) {
        SysMenu sysMenu = new SysMenu();
        BeanUtils.copyProperties(menuDto, sysMenu);
        if (!updateById(sysMenu)) {
            throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
        }
        saveMenuTenant(sysMenu.getId(), menuDto.getTenantIds());
    }

    public void saveMenuTenant(Long menuId, List<Long> tenantIds) {
        if (menuId == null || CollectionUtils.isEmpty(tenantIds)) {
            return;
        }
        //删除关联公司租户
        menuTenantService.remove(Wrappers.<SysMenuTenant>lambdaQuery().eq(SysMenuTenant::getMenuId, menuId));
        //添加新关联
        for (Long tenantId : tenantIds) {
            SysMenuTenant sysMenuTenant = new SysMenuTenant();
            sysMenuTenant.setMenuId(menuId);
            sysMenuTenant.setTenantId(tenantId);
            menuTenantService.save(sysMenuTenant);
        }
    }

    @Override
    public List<SysMenu> getUserNameMenu(SysMenu sysMenu, String username) {
        //获取用户ID
        SysUser sysUser = sysUserService.getOne(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getUsername, username));

        List<SysMenu> menus = new ArrayList<>();
        //筛选条件type
        List<Integer> types = new ArrayList<>();
        if (sysMenu != null && StringUtils.hasText(sysMenu.getTypes())) {
            types = Arrays.stream(sysMenu.getTypes().split(","))
                    .map(value -> Integer.parseInt(value.trim())).collect(Collectors.toList());
        }
        //超级管理员可获得所有菜单权限
        if (sysUser.getUsername().equals(SysConstants.ADMIN)) {
            menus = list(Wrappers.query(sysMenu).lambda().in(!CollectionUtils.isEmpty(types), SysMenu::getType, types)
                    .orderByAsc(SysMenu::getSort));
        } else if (sysUser.getUserLevel() != null && (sysUser.getUserLevel().equals(SysConstants.USER_LEVEL_2) || sysUser.getUserLevel().equals(SysConstants.USER_LEVEL_0))) {
            menus = baseMapper.selectListByUserIdAndTenantId(sysUser.getId(), sysUser.getTenantId(), sysMenu, types);
        } else {
            menus = baseMapper.selectListByUserId(sysUser.getId(), sysMenu, types);
        }
        if (CollectionUtils.isEmpty(menus)) {
            return Collections.emptyList();
        }
        return menus;
    }

    @Override
    public List<SysMenu> listMenu(SysMenu sysMenu) {
        //获取用户ID
        Long userId = SecurityUtils.getUserId();
        if (userId == null) {
            throw new RuntimeException("用户登录信息失效");
        }
        SysUser sysUser = sysUserService.getById(userId);

        List<SysMenu> menus = null;
        //筛选条件type
        List<Integer> types = new ArrayList<>();
        if (sysMenu != null && StringUtils.hasText(sysMenu.getTypes())) {
            types = Arrays.stream(sysMenu.getTypes().split(","))
                    .map(value -> Integer.parseInt(value.trim())).collect(Collectors.toList());
        }
        //超级管理员可获得所有菜单权限
        if (sysUser.getUsername().equals(SysConstants.ADMIN)) {
            menus = list(Wrappers.query(sysMenu).lambda().in(!CollectionUtils.isEmpty(types), SysMenu::getType, types)
                    .orderByAsc(SysMenu::getSort));
        } else if (sysUser.getUserLevel() != null && (sysUser.getUserLevel().equals(SysConstants.USER_LEVEL_2) || sysUser.getUserLevel().equals(SysConstants.USER_LEVEL_0))) {
            menus = baseMapper.selectListByUserIdAndTenantId(userId, sysUser.getTenantId(), sysMenu, types);
        } else {
            menus = baseMapper.selectListByUserId(userId, sysMenu, types);
        }
        if (CollectionUtils.isEmpty(menus)) {
            return Collections.emptyList();
        }
        return menuTenantService.getMenuTenantsByMenuId(menus);
    }

    @Override
    public List<MenuVo> treeMenu(SysMenu sysMenu) {
        List<SysMenu> menus = listMenu(sysMenu);
        return treeMenuList(menus);
    }

    @Transactional
    @Override
    public void delete(List<Long> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            return;
        }
        for (Long id : idList) {
            if (count(Wrappers.<SysMenu>lambdaQuery().eq(SysMenu::getParentId, id)) != 0) {
                throw new RuntimeException("所选菜单下面包含子集，请先删除子集菜单");
            }
        }
        if (!removeByIds(idList)) {
            throw new RuntimeException(ResultEnum.DELETE_FAIL.getMessage());
        }
        //删除关联公司租户
        menuTenantService.remove(Wrappers.<SysMenuTenant>lambdaQuery().in(SysMenuTenant::getMenuId, idList));
        //删除关联的角色
        roleMenuService.remove(Wrappers.<SysRoleMenu>lambdaQuery().in(SysRoleMenu::getMenuId, idList));
    }

    /**
     * 递归开始
     *
     * @param menus 菜单列表
     * @return 树形菜单
     */
    private List<MenuVo> treeMenuList(List<SysMenu> menus) {
        List<MenuVo> menuVos = menus.stream().map(sysMenu -> {
            MenuVo menuVo = new MenuVo();
            BeanUtils.copyProperties(sysMenu, menuVo);
            return menuVo;
        }).collect(Collectors.toList());

        List<MenuVo> menuVoList = new ArrayList<>();
        for (MenuVo menuVo : menuVos) {
            if (menuVo.getParentId().equals(SysConstants.PID)) {
                menuVoList.add(selectChild(menuVo, menuVos));
            }
        }
        return menuVoList;
    }

    /**
     * 递归方法
     *
     * @param menuVo 父节点
     * @param menus  菜单集合
     * @return 树形单节点列表
     */
    private MenuVo selectChild(MenuVo menuVo, List<MenuVo> menus) {
        menuVo.setChildren(new ArrayList<>());
        for (MenuVo menuVoChild : menus) {
            if (menuVo.getChildren() == null) {
                menuVo.setChildren(new ArrayList<>());
            }
            if (menuVo.getId().equals(menuVoChild.getParentId())) {
                menuVo.getChildren().add(selectChild(menuVoChild, menus));
            }
        }
        return menuVo;
    }
}




