package com.tofly.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.haiqiu.tools.excel.utils.HqExcelUtils;
import com.tofly.common.constant.SysConstants;
import com.tofly.common.core.result.ResultEnum;
import com.tofly.common.core.util.PasswordUtil;
import com.tofly.common.ftp.config.FtpUtil;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.system.entity.*;
import com.tofly.system.entity.dto.UserDto;
import com.tofly.system.entity.query.UserQuery;
import com.tofly.system.entity.vo.user.UserExcelExportVo;
import com.tofly.system.entity.vo.user.UserVo;
import com.tofly.system.entity.vo.user.UserVoExcel;
import com.tofly.system.service.*;
import com.tofly.system.mapper.SysUserMapper;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author haiqiu
 * @description 针对表【SYS_USER(系统用户)】的数据库操作Service实现
 * @createDate 2022-07-29 14:23:51
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser>
        implements SysUserService {

    @Value("${user.avatar}")
    private String avatarPath;

    @Autowired
    private SysUserDeptService userDeptService;
    @Autowired
    private SysUserRoleService userRoleService;
    @Autowired
    private SysUserPostService userPostService;
    @Autowired
    private SysRoleService sysRoleService;
    @Autowired
    private SysDeptService sysDeptService;
    @Autowired
    private SysPostService sysPostService;
    @Autowired
    private SysRolePermissionService rolePermissionService;

    @Autowired
    private SysTenantService sysTenantService;
    @Resource
    private CacheManager cacheManager;

    @Override
    public UserVo get(Long id) {
        SysUser sysUser = getById(id);
        if (ObjectUtils.isEmpty(sysUser)) {
            return null;
        }
        UserVo userVo = new UserVo();
        BeanUtils.copyProperties(sysUser, userVo);
        //查询绑定角色列表
        List<SysUserRole> userRoles = userRoleService.list(Wrappers.<SysUserRole>lambdaQuery()
                .eq(SysUserRole::getUserId, userVo.getId()));
        Set<Long> roleIds = userRoles.stream().filter(sysUserRole -> sysUserRole.getRoleId() != null)
                .map(SysUserRole::getRoleId).collect(Collectors.toSet());
        if (!CollectionUtils.isEmpty(roleIds)) {
            List<SysRole> roles = sysRoleService.list(Wrappers.<SysRole>lambdaQuery().in(SysRole::getId, roleIds));
            userVo.setRoles(roles);
        }
        //查询绑定角色列表的所属权限范围
        List<SysRolePermission> sysRolePermissions = rolePermissionService.list(Wrappers.<SysRolePermission>lambdaQuery()
                .in(!CollectionUtils.isEmpty(roleIds), SysRolePermission::getRoleId, roleIds));
        userVo.setSysRolePermissions(sysRolePermissions);
        //查询绑定部门列表
        List<SysUserDept> userDepts = userDeptService.list(Wrappers.<SysUserDept>lambdaQuery()
                .eq(SysUserDept::getUserId, userVo.getId()));
        Set<Long> deptIds = userDepts.stream().filter(sysUserDept -> sysUserDept.getDeptId() != null)
                .map(SysUserDept::getDeptId).collect(Collectors.toSet());
        if (!CollectionUtils.isEmpty(deptIds)) {
            List<SysDept> depts = sysDeptService.list(Wrappers.<SysDept>lambdaQuery().in(SysDept::getId, deptIds));
            userVo.setDepts(depts);
        }
        //查询绑定岗位列表
        List<SysUserPost> userPosts = userPostService.list(Wrappers.<SysUserPost>lambdaQuery()
                .eq(SysUserPost::getUserId, userVo.getId()));
        Set<Long> postIds = userPosts.stream().filter(sysUserPost -> sysUserPost.getPostId() != null)
                .map(SysUserPost::getPostId).collect(Collectors.toSet());
        if (!CollectionUtils.isEmpty(postIds)) {
            List<SysPost> posts = sysPostService.list(Wrappers.<SysPost>lambdaQuery().in(SysPost::getId, postIds));
            userVo.setPosts(posts);
        }
        return userVo;
    }

    @Override
    @Transactional
    public void insert(UserDto userDto, MultipartFile file) {
        //检测用户名是否存在
        checkUserName(userDto);
        SysUser sysUser = new SysUser();
        BeanUtils.copyProperties(userDto, sysUser);
        //TODO 头像上传
        uploadAvatar(file, sysUser);
        //密码加密
        setPwd(userDto, sysUser);
        //保存
        if (!save(sysUser)) {
            throw new RuntimeException(ResultEnum.ADD_FAIL.getMessage());
        }
        //关联角色
        saveRoles(sysUser.getId(), userDto.getRoleIds());
        //关联部门
        saveDepts(sysUser.getId(), userDto.getDeptIds());
        //关联岗位
        savePosts(sysUser.getId(), userDto.getPostIds());
    }

    private void checkUserName(UserDto userDto) {
        if (userDto != null && userDto.getUsername() != null) {
            //携带ID就是修改
            if (userDto.getId() != null) {
                if (count(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getUsername, userDto.getUsername())
                        .ne(SysUser::getId, userDto.getId())) != 0) {
                    throw new RuntimeException(userDto.getUsername() + "登录名已存在");
                }
            }
            //否则就是新增
            else if (count(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getUsername, userDto.getUsername())) != 0) {
                throw new RuntimeException(userDto.getUsername() + "登录名已存在");
            }
        }
    }

    /**
     * 设置密码
     *
     * @param userDto 新增修改实体
     * @param sysUser 数据库实体对象
     */
    private void setPwd(UserDto userDto, SysUser sysUser) {
        BCryptPasswordEncoder bCryptPasswordEncoder = new BCryptPasswordEncoder();
        if (StringUtils.hasText(userDto.getPassword())) {
            String newPwd = bCryptPasswordEncoder.encode(SysConstants.PWD_BEARER + userDto.getPassword());
            sysUser.setPassword(newPwd);
        } else {
            sysUser.setPassword(SysConstants.getUserDefaultPwd());
        }
    }

    @Override
    @Transactional
    public void edit(UserDto userDto, MultipartFile file) {
        if (userDto == null || userDto.getId() == null) {
            throw new RuntimeException("ID不能为空");
        }
        //检测用户名是否存在
        checkUserName(userDto);
        SysUser sysUser = getById(userDto.getId());
        BeanUtils.copyProperties(userDto, sysUser);
        //TODO 头像上传
        uploadAvatar(file, sysUser);
        //修改用户-密码不修改
        sysUser.setPassword(null);
        //修改
        if (!updateById(sysUser)) {
            throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
        }
        //关联角色
        saveRoles(sysUser.getId(), userDto.getRoleIds());
        //关联部门
        saveDepts(sysUser.getId(), userDto.getDeptIds());
        //关联岗位
        savePosts(sysUser.getId(), userDto.getPostIds());
        //清空登录缓存
        clearCached(Collections.singletonList(sysUser.getId()));
    }

    private void uploadAvatar(MultipartFile file, SysUser sysUser) {
        if (file != null) {
            String filename = avatarPath + IdWorker.getIdStr() + file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
            boolean b = FtpUtil.uploadFile2Ftp(file, filename);
            if (b) {
                sysUser.setAvatar(filename);
            }
        }
    }

    @Override
    @Transactional
    public void delete(List<Long> idList) {
        //获取当前登录的账号ID
        Long userId = SecurityUtils.getUserId();
        //判断不能删除自己
        if (!CollectionUtils.isEmpty(idList)) {
            for (Long id : idList) {
                if (userId.equals(id)) {
                    throw new RuntimeException("删除用户包含自己");
                }
            }
        }

        //清空登录缓存
        clearCached(idList);
        if (!removeByIds(idList)) {
            throw new RuntimeException(ResultEnum.DELETE_FAIL.getMessage());
        }
        //删除关联角色
        userRoleService.remove(Wrappers.<SysUserRole>lambdaQuery()
                .in(!CollectionUtils.isEmpty(idList), SysUserRole::getUserId, idList));
        //删除关联部门
        userDeptService.remove(Wrappers.<SysUserDept>lambdaQuery()
                .in(!CollectionUtils.isEmpty(idList), SysUserDept::getUserId, idList));
        //删除关联岗位
        userPostService.remove(Wrappers.<SysUserPost>lambdaQuery()
                .in(!CollectionUtils.isEmpty(idList), SysUserPost::getUserId, idList));
    }

    /**
     * 清空登录缓存
     *
     * @param userIds 用户ID
     */
    @Override
    public void clearCached(List<Long> userIds) {
        //清空缓存
        List<SysUser> users = list(Wrappers.<SysUser>lambdaQuery().in(SysUser::getId, userIds));
        if (!CollectionUtils.isEmpty(users)) {
            for (SysUser user : users) {
                Cache cache = cacheManager.getCache(SysConstants.USER_DATA_INFO);
                cache.put(user.getUsername(), null);
            }
        }
    }

    @Override
    public void template() {
        UserVoExcel voExcel = new UserVoExcel();
        voExcel.setAliasName("测试用户");
        voExcel.setDetails("我是一位测试账号用户");
        voExcel.setDepts("1572145011712598018,1574210413519302657");
        voExcel.setJobNumber("hx123456");
        voExcel.setNote("无备注");
        voExcel.setEmail("xxx@qq.com");
        voExcel.setPhone("13111221122");
        voExcel.setRealName("张三");
        voExcel.setUsername("zhangsan");
        voExcel.setUserLevel(1);
        voExcel.setRoles("1573136814960132097,1573148331310891010");
        voExcel.setPosts("1569856511973859329,1571674503216893954");
        voExcel.setTenantId("1569602913205559298");
        HqExcelUtils.createTemplate("用户导入", "用户列表",
                UserVoExcel.class, Collections.singletonList(voExcel));
    }

    @Override
    public void importData(MultipartFile file) {
        List<UserVoExcel> userVoExcels = HqExcelUtils.getExcelToList("用户列表", file, UserVoExcel.class);
        if (!CollectionUtils.isEmpty(userVoExcels)) {
            for (UserVoExcel voExcel : userVoExcels) {
                UserDto userDto = new UserDto();
                BeanUtils.copyProperties(voExcel, userDto);
                userDto.setTenantId(Long.valueOf(voExcel.getTenantId()));
                //角色赋值导入
                if (StringUtils.hasText(voExcel.getRoles())) {
                    if (voExcel.getRoles().contains(",")) {
                        String[] split = voExcel.getRoles().split(",");
                        List<Long> roleIds = Arrays.stream(split).map(Long::valueOf).collect(Collectors.toList());
                        userDto.setRoleIds(roleIds);
                    } else {
                        userDto.setRoleIds(Collections.singletonList(Long.valueOf(voExcel.getRoles())));
                    }
                }
                //部门赋值导入
                if (StringUtils.hasText(voExcel.getDepts())) {
                    if (voExcel.getDepts().contains(",")) {
                        String[] split = voExcel.getDepts().split(",");
                        List<Long> deptIds = Arrays.stream(split).map(Long::valueOf).collect(Collectors.toList());
                        userDto.setDeptIds(deptIds);
                    } else {
                        userDto.setDeptIds(Collections.singletonList(Long.valueOf(voExcel.getDepts())));
                    }
                }
                //岗位赋值导入
                if (StringUtils.hasText(voExcel.getPosts())) {
                    if (voExcel.getPosts().contains(",")) {
                        String[] split = voExcel.getPosts().split(",");
                        List<Long> postIds = Arrays.stream(split).map(Long::valueOf).collect(Collectors.toList());
                        userDto.setPostIds(postIds);
                    } else {
                        userDto.setPostIds(Collections.singletonList(Long.valueOf(voExcel.getPosts())));
                    }
                }
                //新增
                insert(userDto, null);
            }
        }
    }

    @Override
    public List<SysRole> getRoleByUserId(Long userId) {
        List<SysUserRole> userRoles = userRoleService.list(Wrappers.<SysUserRole>lambdaQuery().eq(SysUserRole::getUserId, userId));
        if (CollectionUtils.isEmpty(userRoles)) {
            return Collections.emptyList();
        }
        List<Long> roleIds = userRoles.stream().map(SysUserRole::getRoleId).collect(Collectors.toList());
        return sysRoleService.list(Wrappers.<SysRole>lambdaQuery().in(SysRole::getId, roleIds));
    }

    @Override
    public List<SysPost> getPostByUserId(Long userId) {
        List<SysUserPost> userPosts = userPostService.list(Wrappers.<SysUserPost>lambdaQuery().eq(SysUserPost::getUserId, userId));
        if (CollectionUtils.isEmpty(userPosts)) {
            return Collections.emptyList();
        }
        List<Long> postIds = userPosts.stream().map(SysUserPost::getPostId).collect(Collectors.toList());
        return sysPostService.list(Wrappers.<SysPost>lambdaQuery().in(SysPost::getId, postIds));
    }

    @Override
    public List<SysDept> getDeptByUserId(Long userId) {
        List<SysUserDept> userDepts = userDeptService.list(Wrappers.<SysUserDept>lambdaQuery().eq(SysUserDept::getUserId, userId));
        if (CollectionUtils.isEmpty(userDepts)) {
            return Collections.emptyList();
        }
        List<Long> deptIds = userDepts.stream().map(SysUserDept::getDeptId).collect(Collectors.toList());
        return sysDeptService.list(Wrappers.<SysDept>lambdaQuery().in(SysDept::getId, deptIds));
    }

    @Override
    public List<UserVo> allList(UserQuery userQuery) {
        List<UserVo> userVos = baseMapper.pageList(getQueryWrapper(userQuery));
        if (!CollectionUtils.isEmpty(userVos)) {
            for (UserVo record : userVos) {
                record.setRoles(getRoleByUserId(record.getId()));
                record.setDepts(getDeptByUserId(record.getId()));
                record.setPosts(getPostByUserId(record.getId()));
            }
        }
        return userVos;
    }

    @Override
    public Page<UserVo> pageList(Page page, UserQuery userQuery) {
        Page<UserVo> userVoPage = baseMapper.pageList(page, getQueryWrapper(userQuery));
        if (!CollectionUtils.isEmpty(userVoPage.getRecords())) {
            userVoPage.setTotal(baseMapper.countList(getQueryWrapper(userQuery)));
            for (UserVo record : userVoPage.getRecords()) {
                record.setRoles(getRoleByUserId(record.getId()));
                record.setDepts(getDeptByUserId(record.getId()));
                record.setPosts(getPostByUserId(record.getId()));
            }
        }
        return userVoPage;
    }


    /**
     * 查询帅选条件
     *
     * @param userQuery 搜索
     * @return 查询条件
     */
    public QueryWrapper<UserQuery> getQueryWrapper(UserQuery userQuery) {
        //查询当前公司和下级公司的人员
        SysTenant sysTenant = new SysTenant();
        sysTenant.setId(userQuery.getTenantIdChildren());
        List<SysTenant> sysTenants = sysTenantService.allList(sysTenant);
        List<Long> tenantIds = new ArrayList<>();
        if (!CollectionUtils.isEmpty(sysTenants)) {
            tenantIds = sysTenants.stream().map(SysTenant::getId).collect(Collectors.toList());
        }
        QueryWrapper<UserQuery> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.hasText(userQuery.getUsername()), "u.USERNAME", userQuery.getUsername())
                .like(StringUtils.hasText(userQuery.getRealName()), "u.REAL_NAME", userQuery.getRealName())
                .like(StringUtils.hasText(userQuery.getPhone()), "u.PHONE", userQuery.getPhone())
                .like(StringUtils.hasText(userQuery.getJobNumber()), "u.JOB_NUMBER", userQuery.getJobNumber())
                .in(!CollectionUtils.isEmpty(userQuery.getDepts()), "d.ID", userQuery.getDepts())
                .in(!CollectionUtils.isEmpty(userQuery.getPosts()), "p.ID", userQuery.getPosts())
                .in(!CollectionUtils.isEmpty(userQuery.getRoles()), "r.ID", userQuery.getRoles())
                .eq(userQuery.getActive() != null, "u.ACTIVE", userQuery.getActive())
                .eq(userQuery.getLockFlag() != null, "u.LOCK_FLAG", userQuery.getLockFlag())
                .eq(userQuery.getTenantId() != null, "u.TENANT_ID", userQuery.getTenantId())
                .eq(userQuery.getUserLevel() != null, "u.USER_LEVEL", userQuery.getUserLevel())
                .in(!CollectionUtils.isEmpty(tenantIds), "u.TENANT_ID", tenantIds)
                .like(StringUtils.hasText(userQuery.getTenantName()), "t.NAME", userQuery.getTenantName())
                .in(userQuery.getIsManage()!=null && userQuery.getIsManage(),"u.USER_LEVEL",Arrays.asList(SysConstants.USER_LEVEL_0,SysConstants.USER_LEVEL_2))
                .in(userQuery.getIsManage()!=null && !userQuery.getIsManage(),"u.USER_LEVEL",Arrays.asList(SysConstants.USER_LEVEL_1,SysConstants.USER_LEVEL_3))
                .eq("u.DELETED", SysConstants.NOT_DEL);
        return queryWrapper;
    }

    @Override
    public void export(Page page, UserQuery userQuery, Boolean isAll) {
        List<UserExcelExportVo> userVoExcels = new ArrayList<>();
        Page<UserVo> userVoPage = new Page<>();

        //查询数据
        if (isAll == null || Boolean.FALSE.equals(isAll)) {
            userVoPage = pageList(page, userQuery);
        } else {
            List<UserVo> userVos = baseMapper.pageList(getQueryWrapper(userQuery));
            userVoPage.setRecords(userVos);
            for (UserVo record : userVoPage.getRecords()) {
                record.setRoles(getRoleByUserId(record.getId()));
                record.setDepts(getDeptByUserId(record.getId()));
                record.setPosts(getPostByUserId(record.getId()));
            }
        }
        //导出数据整理
        if (!CollectionUtils.isEmpty(userVoPage.getRecords())) {
            userVoExcels = userVoPage.getRecords().stream().map(userVo -> {
                UserExcelExportVo voExcel = new UserExcelExportVo();
                BeanUtils.copyProperties(userVo, voExcel);
                if (!CollectionUtils.isEmpty(userVo.getDepts())) {
                    List<String> collect = userVo.getDepts().stream().map(SysDept::getName).collect(Collectors.toList());
                    String depts = collect.toString().replace("[", "").replace("]", "");
                    voExcel.setDepts(depts);
                }
                if (!CollectionUtils.isEmpty(userVo.getRoles())) {
                    List<String> collect = userVo.getRoles().stream().map(SysRole::getName).collect(Collectors.toList());
                    String roles = collect.toString().replace("[", "").replace("]", "");
                    voExcel.setRoles(roles);
                }
                if (!CollectionUtils.isEmpty(userVo.getPosts())) {
                    List<String> collect = userVo.getPosts().stream().map(SysPost::getName).collect(Collectors.toList());
                    String posts = collect.toString().replace("[", "").replace("]", "");
                    voExcel.setPosts(posts);
                }
                return voExcel;
            }).collect(Collectors.toList());
        }
        HqExcelUtils.getListToExcel(UserExcelExportVo.class, userVoExcels, "用户列表");
    }

    /**
     * 用户角色关联处理
     *
     * @param userId  用户ID
     * @param roleIds 角色ID集合
     */
    public void saveRoles(Long userId, List<Long> roleIds) {
        if (!CollectionUtils.isEmpty(roleIds)) {
            //删除
            userRoleService.remove(Wrappers.<SysUserRole>lambdaQuery()
                    .eq(userId != null, SysUserRole::getUserId, userId));
            //新增
            List<SysUserRole> userRoles = roleIds.stream().map(roleId -> {
                SysUserRole sysUserRole = new SysUserRole();
                sysUserRole.setUserId(userId);
                sysUserRole.setRoleId(roleId);
                return sysUserRole;
            }).collect(Collectors.toList());
            userRoleService.saveBatch(userRoles);
        }
    }

    /**
     * 用户部门关联处理
     *
     * @param userId  用户ID
     * @param deptIds 部门ID集合
     */
    public void saveDepts(Long userId, List<Long> deptIds) {
        if (!CollectionUtils.isEmpty(deptIds)) {
            //删除
            userDeptService.remove(Wrappers.<SysUserDept>lambdaQuery()
                    .eq(userId != null, SysUserDept::getUserId, userId));
            //新增
            List<SysUserDept> userDepts = deptIds.stream().map(deptId -> {
                SysUserDept sysUserDept = new SysUserDept();
                sysUserDept.setUserId(userId);
                sysUserDept.setDeptId(deptId);
                return sysUserDept;
            }).collect(Collectors.toList());
            userDeptService.saveBatch(userDepts);
        }
    }


    /**
     * 用户岗位关联处理
     *
     * @param userId  用户ID
     * @param postIds 岗位ID集合
     */
    public void savePosts(Long userId, List<Long> postIds) {
        if (!CollectionUtils.isEmpty(postIds)) {
            //删除
            userPostService.remove(Wrappers.<SysUserPost>lambdaQuery()
                    .eq(userId != null, SysUserPost::getUserId, userId));
            //新增
            List<SysUserPost> userPosts = postIds.stream().map(postId -> {
                SysUserPost sysUserPost = new SysUserPost();
                sysUserPost.setUserId(userId);
                sysUserPost.setPostId(postId);
                return sysUserPost;
            }).collect(Collectors.toList());
            userPostService.saveBatch(userPosts);
        }
    }

    @Override
    public void updatePwd(String oldPwd, String newPwd, String newRePwd) {
        Long userId = SecurityUtils.getUserId();
        SysUser sysUser = getById(userId);
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
//        if (!passwordEncoder.matches(SysConstants.PWD_BEARER + oldPwd, sysUser.getPassword())) {
//            throw new RuntimeException("原密码不正确");
//        }
        if (!newPwd.equals(newRePwd)) {
            throw new RuntimeException("新密码两次输入不一致");
        }
        sysUser.setPassword(passwordEncoder.encode(SysConstants.PWD_BEARER + newPwd));
        sysUser.setFirstLogin(0);
        if (!updateById(sysUser)) {
            throw new RuntimeException(ResultEnum.UPDATE_FAIL.getMessage());
        }
        //清空登录缓存
        clearCached(Collections.singletonList(sysUser.getId()));
    }


}